# -*- coding: utf-8 -*-
from plone.app.blocks.utils import resolve

import unittest


class TestUtils(unittest.TestCase):

    def test_resolve_utf8_unicode(self):
        content_layout = u"""
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html data-layout="./@@default-site-layout">
  <body>
    <h1>Ä</h1>
  </body>
</html>"""
        text = resolve('', content_layout).xpath('//h1')[0].text
        self.assertEqual(u'Ä', text)

    def test_resolve_utf8_bytestring(self):
        """Test fix for issue where layouts with non-ascii characters were
        not properly parsed resulting in double encoding
        """
        content_layout = u"""
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html data-layout="./@@default-site-layout">
  <body>
    <h1>Ä</h1>
  </body>
</html>""".encode('utf-8')
        text = resolve('', content_layout).xpath('//h1')[0].text
        self.assertEqual(u'Ä', text)



# -*- coding: utf-8 -*-
from lxml import html
from plone.app.blocks.testing import BLOCKS_FUNCTIONAL_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.registry.interfaces import IRegistry
from plone.tiles.data import ANNOTATIONS_KEY_PREFIX
from zope.annotation.interfaces import IAnnotations
from zope.component import getGlobalSiteManager
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory

import pkg_resources
import unittest


try:
    pkg_resources.get_distribution('plone.app.contenttypes')
except pkg_resources.DistributionNotFound:
    HAS_PLONE_APP_CONTENTTYPES = False
else:
    HAS_PLONE_APP_CONTENTTYPES = True


class TestContentLayout(unittest.TestCase):

    layer = BLOCKS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.registry = getUtility(IRegistry)

        setRoles(self.portal, TEST_USER_ID, ('Manager',))
        self.portal.invokeFactory('Folder', 'f1', title=u"Folder 1")
        self.portal['f1'].invokeFactory('Document', 'd1', title=u"Document 1")
        setRoles(self.portal, TEST_USER_ID, ('Member',))

        if HAS_PLONE_APP_CONTENTTYPES:
            from plone.app.contenttypes.interfaces import IDocument
            iface = IDocument
        else:
            iface = self.portal['f1']['d1'].__class__

        from plone.app.blocks.layoutbehavior import ILayoutAware
        from plone.app.blocks.layoutbehavior import LayoutAwareBehavior

        sm = getGlobalSiteManager()
        sm.registerAdapter(LayoutAwareBehavior, [iface])
        registrations = sm.getAdapters(
            (self.portal['f1']['d1'],),
            ILayoutAware
        )
        self.assertEqual(len(list(registrations)), 1)
        self.behavior = ILayoutAware(self.portal['f1']['d1'])

    def test_content_layout_vocabulary(self):
        factory = getUtility(IVocabularyFactory,
                             name='plone.availableContentLayouts')
        vocab = factory(self.layer['portal'])
        self.assertEqual(len(vocab), 3)

        self.assertIn('testlayout1/content.html', vocab.by_token)
        self.assertEqual(
            vocab.getTermByToken('testlayout1/content.html').value,
            u'/++contentlayout++testlayout1/content.html')
        self.assertEqual(
            vocab.getTermByToken('testlayout1/content.html').title,
            'Testlayout1')

        self.assertIn('testlayout2/mylayout.html', vocab.by_token)
        self.assertEqual(
            vocab.getTermByToken('testlayout2/mylayout.html').value,
            u'/++contentlayout++testlayout2/mylayout.html')
        self.assertEqual(
            vocab.getTermByToken('testlayout2/mylayout.html').title,
            'My content layout')

        self.assertIn('testlayout2/mylayout2.html', vocab.by_token)
        self.assertEqual(
            vocab.getTermByToken('testlayout2/mylayout2.html').value,
            u'/++contentlayout++testlayout2/mylayout2.html')
        self.assertEqual(
            vocab.getTermByToken('testlayout2/mylayout2.html').title,
            'My content layout 2')

    def test_content_layout(self):
        from plone.app.blocks.layoutbehavior import ContentLayoutView
        from plone.app.blocks.utils import bodyTileXPath
        from plone.app.blocks.utils import tileAttrib
        self.behavior.contentLayout = \
            '/++contentlayout++testlayout1/content.html'
        rendered = ContentLayoutView(self.portal['f1']['d1'], self.request)()
        tree = html.fromstring(rendered)
        tiles = [node.attrib[tileAttrib] for node in bodyTileXPath(tree)]
        self.assertIn(
            './@@test.tile1/tile2?magicNumber:int=2&X-Tile-Persistent=yes',
            tiles)
        self.assertIn(
            './@@test.tile1/tile3?X-Tile-Persistent=yes',
            tiles)

    def test_error_layout(self):
        from plone.app.blocks.layoutbehavior import ContentLayoutView
        self.behavior.contentLayout = \
            '/++sitelayout++missing/missing.html'
        rendered = ContentLayoutView(self.portal['f1']['d1'], self.request)()
        self.assertIn('Could not find layout for content', rendered)

    def test_getLayout(self):
        from plone.app.blocks.utils import getLayout
        self.behavior.contentLayout = \
            '/++contentlayout++testlayout1/content.html'
        layout = getLayout(self.portal['f1']['d1'])
        self.assertIn(
            './@@test.tile1/tile2?magicNumber:int=2',
            layout)

    def test_getLayout_custom(self):
        self.behavior.customContentLayout = """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html data-layout="./@@default-site-layout">
  <body>
    <h1>Foobar!</h1>
    <div data-panel="panel1">
      Page panel 1
       <div id="page-tile2" data-tile="./@@test.tile1/tile99?magicNumber:int=3">
       Page tile 2 placeholder</div>
    </div>
  </body>
</html>"""  # noqa
        from plone.app.blocks.utils import getLayout
        layout = getLayout(self.portal['f1']['d1'])
        self.assertIn(
            './@@test.tile1/tile99?magicNumber:int=3',
            layout
        )

    def test_getting_indexed_data(self):
        self.behavior.content = """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html data-layout="./@@default-site-layout">
<body>
  <div data-tile="./@@plone.app.standardtiles.html/example"
    <p>Foobar inserted text tile</p>
  </div>
</body>
</html>"""
        self.behavior.customContentLayout = """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html data-layout="./@@default-site-layout">
<body>
  <div class="mosaic-tile rawhtml-tile
              mosaic-plone.app.standardtiles.rawhtml-tile">
    <div class="mosaic-tile-content">
      <div data-tile="./@@plone.app.standardtiles.rawhtml/rawhtml-1"></div>
    </div>
  </div>
  <div class="mosaic-tile mosaic-text-tile">
    <div class="mosaic-tile-content">
      <div data-tile="./@@plone.app.standardtiles.html/example"></div>
    </div>
  </div>
</body>
</html>"""
        obj = self.portal['f1']['d1']
        annotations = IAnnotations(obj)
        annotations[ANNOTATIONS_KEY_PREFIX + '.rawhtml-1'] = {
            'content': '<p>Foobar inserted raw tile</p>'
        }
        from plone.app.blocks.indexing import LayoutSearchableText
        indexed_data = LayoutSearchableText(obj)()
        self.assertTrue('Foobar inserted text tile' in indexed_data)
        self.assertTrue('Foobar inserted raw tile' in indexed_data)

    def test_on_save_tile_data_is_cleaned(self):
        self.behavior.customContentLayout = """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html data-layout="./@@default-site-layout">
<body>
  <div data-tile="./@@faketile/foobar-1"></div>
  <div data-tile="./@@faketile/foobar-2"></div>
  <div data-tile="./@@faketile/foobar-3"></div>
  <div data-tile="./@@faketile/foobar-4"></div>
</body>
</html>"""

        obj = self.portal['f1']['d1']
        annotations = IAnnotations(obj)
        annotations.update({
            ANNOTATIONS_KEY_PREFIX + '.foobar-1': {
                'foo': 'bar'
            },
            ANNOTATIONS_KEY_PREFIX + '.foobar-2': {
                'foo': 'bar'
            },
            ANNOTATIONS_KEY_PREFIX + '.foobar-3': {
                'foo': 'bar'
            },
            ANNOTATIONS_KEY_PREFIX + '.foobar-4': {
                'foo': 'bar'
            },
            ANNOTATIONS_KEY_PREFIX + '.bad-1': {
                'foo': 'bar'
            },
            ANNOTATIONS_KEY_PREFIX + '.bad-2': {
                'foo': 'bar'
            },
            ANNOTATIONS_KEY_PREFIX + '.bad-3': {
                'foo': 'bar'
            },
        })

        from plone.app.blocks.subscribers import onLayoutEdited
        onLayoutEdited(obj, None)
        annotations = IAnnotations(obj)
        self.assertTrue(ANNOTATIONS_KEY_PREFIX + '.foobar-1' in annotations)
        self.assertTrue(ANNOTATIONS_KEY_PREFIX + '.foobar-2' in annotations)
        self.assertTrue(ANNOTATIONS_KEY_PREFIX + '.foobar-3' in annotations)
        self.assertTrue(ANNOTATIONS_KEY_PREFIX + '.foobar-4' in annotations)
        self.assertFalse(ANNOTATIONS_KEY_PREFIX + '.bad-1' in annotations)
        self.assertFalse(ANNOTATIONS_KEY_PREFIX + '.bad-2' in annotations)
        self.assertFalse(ANNOTATIONS_KEY_PREFIX + '.bad-3' in annotations)



# -*- coding: utf-8 -*-
from plone.app.blocks.interfaces import DEFAULT_SITE_LAYOUT_REGISTRY_KEY
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone.app.blocks.layoutbehavior import LayoutAwareBehavior
from plone.app.blocks.testing import BLOCKS_FUNCTIONAL_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.registry.interfaces import IRegistry
from zExceptions import NotFound
from zope.component import getGlobalSiteManager
from zope.component import getMultiAdapter
from zope.component import getUtility
import pkg_resources
import transaction
import unittest

try:
    pkg_resources.get_distribution('plone.app.contenttypes')
except pkg_resources.DistributionNotFound:
    HAS_PLONE_APP_CONTENTTYPES = False
else:
    HAS_PLONE_APP_CONTENTTYPES = True


class TestPageSiteLayout(unittest.TestCase):

    layer = BLOCKS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.registry = getUtility(IRegistry)

        setRoles(self.portal, TEST_USER_ID, ('Manager',))
        self.portal.invokeFactory('Folder', 'f1', title=u"Folder 1")
        self.portal['f1'].invokeFactory('Document', 'd1', title=u"Document 1")
        setRoles(self.portal, TEST_USER_ID, ('Member',))

        # setup default behavior
        self.registry[DEFAULT_SITE_LAYOUT_REGISTRY_KEY] =\
            '/++sitelayout++testlayout1/site.html'
        iface = self.portal['f1']['d1'].__class__
        sm = getGlobalSiteManager()
        sm.registerAdapter(LayoutAwareBehavior, [iface])
        registrations = sm.getAdapters(
            (self.portal['f1']['d1'],),
            ILayoutAware
        )
        self.assertEqual(len(list(registrations)), 1)
        self.behavior = ILayoutAware(self.portal['f1']['d1'])
        registrations = sm.getAdapters(
            (self.portal['f1']['d1'],),
            ILayoutAware
        )
        self.assertEqual(len(list(registrations)), 1)

    def test_page_site_layout_no_registry_key(self):
        self.registry[DEFAULT_SITE_LAYOUT_REGISTRY_KEY] = None
        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        self.assertRaises(NotFound, view.index)

        from plone.app.blocks.layoutbehavior import SiteLayoutView
        default_view = SiteLayoutView(self.portal['f1']['d1'], self.request)
        self.assertEqual(view().split(), default_view().split())

    def test_page_site_layout_default(self):
        self.registry[DEFAULT_SITE_LAYOUT_REGISTRY_KEY] =\
            '/++sitelayout++testlayout1/site.html'
        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()
        self.assertTrue(u"Layout title" in rendered)

    def test_page_site_layout_page_override(self):
        self.behavior.customContentLayout = u"<html><body>N/A</body></html>"
        self.behavior.pageSiteLayout = \
            '/++sitelayout++testlayout2/mylayout.html'

        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()
        self.assertFalse(u"Layout title" in rendered)
        self.assertTrue(u"My Layout 1 Title" in rendered)

    def test_page_site_layout_section_override(self):
        self.behavior.sectionSiteLayout = \
            '/++sitelayout++testlayout2/mylayout.html'

        view = getMultiAdapter(
            (self.portal['f1']['d1'], self.request,),
            name=u'page-site-layout'
        )
        rendered = view()

        self.assertFalse(u"Layout title" in rendered)
        self.assertTrue(u"My Layout 1 Title" in rendered)

    def test_page_site_layout_cache(self):
        self.behavior.pageSiteLayout = \
            '/++sitelayout++testlayout2/mylayout.html'

        view = getMultiAdapter(
            (self.portal['f1']['d1'], self.request,),
            name=u'page-site-layout'
        )
        rendered = view()

        self.assertFalse(u"Layout title" in rendered)
        self.assertTrue(u"My Layout 1 Title" in rendered)

        # Change the section value
        self.behavior.pageSiteLayout = \
            '/++sitelayout++testlayout1/site.html'

        view = getMultiAdapter(
            (self.portal['f1']['d1'], self.request,),
            name=u'page-site-layout'
        )
        rendered = view()

        # Cache means our change is ignored
        self.assertFalse(u"Layout title" in rendered)
        self.assertTrue(u"My Layout 1 Title" in rendered)

    def test_page_site_layout_cache_invalidate_mtime(self):
        self.behavior.customContentLayout = u"<html><body>N/A</body></html>"
        self.behavior.sectionSiteLayout = \
            '/++sitelayout++testlayout2/mylayout.html'

        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()

        self.assertFalse(u"Layout title" in rendered)
        self.assertTrue(u"My Layout 1 Title" in rendered)

        # Trigger invalidation by modifying the context
        self.portal['f1']['d1'].title = u"New title"
        transaction.commit()

        # Change the section value
        self.behavior.sectionSiteLayout = \
            '/++sitelayout++testlayout1/site.html'

        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()

        # We now get the new layout
        self.assertTrue(u"Layout title" in rendered)
        self.assertFalse(u"My Layout 1 Title" in rendered)

    def test_page_site_layout_cache_invalidate_catalog_counter(self):
        self.behavior.sectionSiteLayout = \
            '/++sitelayout++testlayout2/mylayout.html'

        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()

        self.assertFalse(u"Layout title" in rendered)
        self.assertTrue(u"My Layout 1 Title" in rendered)

        # Trigger invalidation by incrementing the catalog counter
        self.portal['portal_catalog']._increment_counter()

        # Change the section value
        self.behavior.sectionSiteLayout = \
            '/++sitelayout++testlayout1/site.html'

        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()

        # We now get the new layout
        self.assertTrue(u"Layout title" in rendered)
        self.assertFalse(u"My Layout 1 Title" in rendered)

    def test_page_site_layout_cache_invalidate_registry_key(self):
        self.behavior.sectionSiteLayout = \
            '/++sitelayout++testlayout2/mylayout.html'

        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()

        self.assertFalse(u"Layout title" in rendered)
        self.assertTrue(u"My Layout 1 Title" in rendered)

        # Trigger invalidation by modifying the global registry key
        self.registry[DEFAULT_SITE_LAYOUT_REGISTRY_KEY] =\
            '/++sitelayout++testlayout2/mylayout.html'

        # Change the section value
        self.behavior.sectionSiteLayout = \
            '/++sitelayout++testlayout1/site.html'

        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()

        # We now get the new layout
        self.assertTrue(u"Layout title" in rendered)
        self.assertFalse(u"My Layout 1 Title" in rendered)


class TestPageSiteLayoutAcquisition(unittest.TestCase):

    layer = BLOCKS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.registry = getUtility(IRegistry)

        setRoles(self.portal, TEST_USER_ID, ('Manager',))
        self.portal.invokeFactory('Folder', 'f1', title=u"Folder 1")
        self.portal['f1'].invokeFactory('Document', 'd1', title=u"Document 1")
        setRoles(self.portal, TEST_USER_ID, ('Member',))

        sm = getGlobalSiteManager()

        # setup default behaviors
        self.registry[DEFAULT_SITE_LAYOUT_REGISTRY_KEY] = \
            '/++sitelayout++testlayout1/site.html'

        iface = self.portal['f1'].__class__
        sm.registerAdapter(LayoutAwareBehavior, [iface])

        iface = self.portal['f1']['d1'].__class__
        sm.registerAdapter(LayoutAwareBehavior, [iface])

        registrations = sm.getAdapters(
            (self.portal['f1'],),
            ILayoutAware
        )
        self.assertEqual(len(list(registrations)), 1)

        registrations = sm.getAdapters(
            (self.portal['f1']['d1'],),
            ILayoutAware
        )
        self.assertEqual(len(list(registrations)), 1)

    def test_page_site_layout_is_not_acquired(self):
        self.registry[DEFAULT_SITE_LAYOUT_REGISTRY_KEY] = \
            '/++sitelayout++testlayout1/site.html'

        a1 = ILayoutAware(self.portal['f1'])
        a2 = ILayoutAware(self.portal['f1']['d1'])

        self.assertEqual(a1.site_layout(), a2.site_layout())

        a1.pageSiteLayout = '/++sitelayout++testlayout2/mylayout.html'

        self.assertNotEqual(a1.site_layout(), a2.site_layout())



# -*- coding: utf-8 -*-
from plone.app.blocks.testing import BLOCKS_FUNCTIONAL_TESTING_PRETTY_PRINT
from plone.testing import layered

import doctest
import unittest


optionflags = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE |
               doctest.REPORT_NDIFF)

doc_tests = [
    'context.rst',
    'esi.rst',
    'rendering.rst',
]


def test_suite():
    suite = unittest.TestSuite()
    suite.addTests([
        layered(doctest.DocFileSuite(test_file,
                                     optionflags=optionflags),
                layer=BLOCKS_FUNCTIONAL_TESTING_PRETTY_PRINT)
        for test_file in doc_tests]
    )
    return suite



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup

import os

version = '4.0.5'
widgets_require = [
    'plone.app.widgets'
]
test_require = [
    'plone.app.tiles',
    'plone.app.testing',
    'plone.testing',
    'Products.BTreeFolder2',
    'transaction',
    'zExceptions',
    'zope.configuration',
]


def read(*path):
    return open(os.path.join(*path)).read()

long_description = '\n'.join(
    [
        open('README.rst').read(),
        open(
            os.path.join('plone', 'app', 'blocks', 'tests', 'rendering.rst')
        ).read(),
        open(
            os.path.join('plone', 'app', 'blocks', 'tests', 'esi.rst')
        ).read(),
        open('CHANGES.rst').read(),
    ]
)


setup(
    name='plone.app.blocks',
    version=version,
    description='Implements the in-Plone blocks rendering process',
    long_description=long_description,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Plone',
        'Framework :: Plone :: 4.3',
        'Framework :: Plone :: 5.0',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords='plone blocks deco',
    author='Martin Aspeli, Laurence Rowe',
    author_email='optilude@gmail.com',
    url='https://github.com/plone/plone.app.blocks',
    packages=find_packages(),
    namespace_packages=['plone', 'plone.app'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'AccessControl',
        'Acquisition',
        'diazo',
        'lxml',
        'plone.app.layout',
        'plone.app.registry',
        'plone.autoform',
        'plone.behavior>=1.1',
        'plone.dexterity',
        'plone.jsonserializer',
        'plone.memoize',
        'plone.outputfilters',
        'plone.registry',
        'plone.resource',
        'plone.subrequest',
        'plone.supermodel',
        'plone.tiles',
        'plone.transformchain',
        'plone.uuid',
        'Products.CMFCore',
        'Products.CMFPlone >= 4.3',
        'repoze.xmliter',
        'setuptools',
        'z3c.form',
        'zope.annotation',
        'zope.component',
        'zope.globalrequest',
        'zope.i18nmessageid',
        'zope.interface',
        'zope.publisher',
        'zope.schema',
        'zope.security',
        'zope.site',
        'zope.traversing',
        'Zope2',
    ],
    extras_require={
        'widgets': widgets_require,
        'test': test_require,
    },
    entry_points="""
        [z3c.autoinclude.plugin]
        target = plone
    """,
)
